from PropBank.Role import Role


class RoleSet(object):

    __id: str
    __name: str
    __roles: list

    def __init__(self, _id: str, name: str):
        """
        A constructor of RoleSet class which takes id and name as inputs and initializes corresponding attributes
        with these inputs.

        PARAMETERS
        ----------
        _id : str
            Id of the roleSet
        name : str
            Name of the roleSet
        """
        self.__id = _id
        self.__name = name
        self.__roles = []

    def getId(self) -> str:
        """
        Accessor for id.

        RETURNS
        -------
        str
            id.
        """
        return self.__id

    def getName(self) -> str:
        """
        Accessor for name.

        RETURNS
        -------
        str
            name.
        """
        return self.__name

    def size(self) -> int:
        """
        The size method returns the size of the roles list.

        RETURNS
        -------
        int
            the size of the roles list.
        """
        return len(self.__roles)

    def addRole(self, role: Role):
        """
        The addRole method takes a Role as input and adds it to the roles list.

        PARAMETERS
        ----------
        role : Role
            Role to be added
        """
        self.__roles.append(role)

    def getRole(self, index: int) -> Role:
        """
        The getRole method returns the role at the given index.

        PARAMETERS
        ----------
        index : int
            Index of the role

        RETURNS
        -------
        Role
            Role at the given index.
        """
        return self.__roles[index]

    def getRoleWithArgument(self, n: str) -> Role:
        """
        Finds and returns the role with the given argument number n. For example, if n == 0, the method returns
        the argument ARG0.

        PARAMETERS
        ----------
        n : str
            Argument number

        RETURNS
        -------
        Role
            The role with the given argument number n.
        """
        for role in self.__roles:
            if role.getN() == n:
                return role

    def __repr__(self):
        return f"{self.__id} {self.__name} {self.__roles}"



from __future__ import annotations
from enum import Enum, auto


class ArgumentType(Enum):
    """
    Enumerated class for argument type.
    """
    NONE = auto()
    PREDICATE = auto()
    ARG0 = auto()
    ARG1 = auto()
    ARG2 = auto()
    ARG3 = auto()
    ARG4 = auto()
    ARG5 = auto()
    ARGMNONE = auto()
    ARGMEXT = auto()
    ARGMLOC = auto()
    ARGMDIS = auto()
    ARGMADV = auto()
    ARGMCAU = auto()
    ARGMTMP = auto()
    ARGMPNC = auto()
    ARGMMNR = auto()
    ARGMDIR = auto()

    @staticmethod
    def getArguments(argumentsType: str) -> ArgumentType:
        """
        The getArguments method takes an argumentsType string and returns the ArgumentType form of it.

        PARAMETERS
        ----------
        argumentsType : str
            Type of the argument in string form

        RETURNS
        -------
        ArgumentType
            Type of the argument in ArgumentType form
        """
        for argument_type in ArgumentType:
            if argumentsType.upper() == argument_type.name:
                return argument_type
        return ArgumentType.NONE

    @staticmethod
    def getPropbankType(argumentType: ArgumentType) -> str:
        """
        The getPropbankType method takes an argumentType in ArgumentType form and returns the string form of it.

        PARAMETERS
        ----------
        argumentType : ArgumentType
            Type of the argument in {@link ArgumentType} form

        RETURNS
        -------
        str
            Type of the argument in string form
        """
        if argumentType is None:
            return "NONE"
        return argumentType.name



import xml

import pkg_resources

from PropBank.Frameset import Frameset
import xml.etree.ElementTree


class FramesetList(object):

    __frames: list

    def __init__(self, fileName = pkg_resources.resource_filename(__name__, 'data/turkish-propbank.xml')):
        """
        A constructor of FramesetList class which reads all frameset files inside the Predicates folder. For each
        file inside that folder, the constructor creates a Frameset and puts in inside the frames list.
        """
        self.__frames = []
        root = xml.etree.ElementTree.parse(fileName).getroot()
        for frameset_node in root:
            frameset = Frameset(frameset_node)
            self.__frames.append(frameset)

    def readFromXml(self, synSetId: str) -> dict:
        """
        readFromXmL method searches the Frameset with a given synSetId if there is a Frameset with the given synSet id,
        returns the arguments of that Frameset as a dictionary.

        PARAMETERS
        ----------
        synSetId : str
            Id of the searched Frameset

        RETURNS
        -------
        dict
            a dict containing the arguments of the searched Frameset
        """
        frameset = {}
        for f in self.__frames:
            if f.getId() == synSetId:
                for i in range(len(f.getFramesetArguments())):
                    frameset_argument = f.getFramesetArguments()[i]
                    frameset[frameset_argument.getArgumentType()] = frameset_argument.getDefinition()
        return frameset

    def frameExists(self, synSetId: str) -> bool:
        """
        frameExists method checks if there is a Frameset with the given synSet id.

        PARAMETERS
        ----------
        synSetId : str
            Id of the searched Frameset

        RETURNS
        -------
        bool
            true if the Frameset with the given id exists, false otherwise.
        """
        for f in self.__frames:
            if f.getId() == synSetId:
                return True
        return False

    def getFrameSet(self, synSetIdOrIndex) -> Frameset:
        """
        getFrameSet method returns the Frameset with the given synSet id or index

        PARAMETERS
        ----------
        synSetIdOrIndex
            Id of the searched Frameset

        RETURNS
        -------
        Frameset
            Frameset which has the given id.
        """
        if isinstance(synSetIdOrIndex, str):
            for f in self.__frames:
                if f.getId() == synSetIdOrIndex:
                   return f
        elif isinstance(synSetIdOrIndex, int):
            return self.__frames[synSetIdOrIndex]
        return None

    def addFrameset(self, frameset: Frameset):
        """
        The addFrameset method takes a Frameset as input and adds it to the frames list.

        PARAMETERS
        ----------
        frameset : Frameset
            Frameset to be added
        """
        self.__frames.append(frameset)

    def size(self) -> int:
        """
        The size method returns the size of the frames list.

        RETURNS
        -------
        int
            the size of the frames list.
        """
        return len(self.__frames)



from PropBank.Argument import Argument


class ArgumentList:

    arguments: [Argument]

    def __init__(self, argumentList: str):
        """
        Constructor of argument list from a string. The arguments for a word is a concatenated list of arguments
        separated via '#' character.
        :param argumentList: String consisting of arguments separated with '#' character.
        """
        self.arguments = []
        items = argumentList.split('#')
        for item in items:
            self.arguments.append(Argument(item))

    def __str__(self) -> str:
        """
        Overloaded toString method to convert an argument list to a string. Concatenates the string forms of all
        arguments with '#' character.
        :return: String form of the argument list.
        """
        if len(self.arguments) == 0:
            return "NONE"
        else:
            result = self.arguments[0].__str__()
            for i in range(1, len(self.arguments)):
                result += "#" + self.arguments[i].__str__()
            return result

    def updateConnectedId(self, previousId: str, currentId: str):
        """
        Replaces id's of predicates, which have previousId as synset id, with currentId.
        :param previousId: Previous id of the synset.
        :param currentId: Replacement id.
        """
        for argument in self.arguments:
            if argument.getId() == previousId:
                argument.setId(currentId)

    def addPredicate(self, predicateId: str):
        """
        Adds a predicate argument to the argument list of this word.
        :param predicateId: Synset id of this predicate.
        """
        if len(self.arguments) != 0 and self.arguments[0].getArgumentType() == "NONE":
            self.arguments.pop(0)
        self.arguments.append(Argument("PREDICATE", predicateId))

    def removePredicate(self):
        """
        Removes the predicate with the given predicate id.
        """
        for argument in self.arguments:
            if argument.getArgumentType() == "PREDICATE":
                self.arguments.remove(argument)
                break

    def containsPredicate(self) -> bool:
        """
        Checks if one of the arguments is a predicate.
        :return: True, if one of the arguments is predicate; false otherwise.
        """
        for argument in self.arguments:
            if argument.getArgumentType() == "PREDICATE":
                return True
        return False

    def containsPredicateWithId(self, predicateId: str) -> bool:
        """
        Checks if one of the arguments is a predicate with the given id.
        :param predicateId: Synset id to check.
        :return: True, if one of the arguments is predicate; false otherwise.
        """
        for argument in self.arguments:
            if argument.getArgumentType() == "PREDICATE" and argument.getId() == predicateId:
                return True
        return False

    def getArguments(self) -> [str]:
        """
        Returns the arguments as an array list of strings.
        :return: Arguments as an array list of strings.
        """
        result = []
        for argument in self.arguments:
            result.append(argument.__str__())
        return result

    def containsArgument(self, argumentType: str, _id: str) -> bool:
        """
        Checks if the given argument with the given type and id exists or not.
        :param argumentType: Type of the argument to search for.
        :param _id: Id of the argument to search for.
        :return: True if the argument exists, false otherwise.
        """
        for argument in self.arguments:
            if argument.getArgumentType() == argumentType and argument.getId() == _id:
                return True
        return False



class FramesetArgument(object):

    __argument_type: str
    __definition: str
    __function: str

    def __init__(self,
                 argumentType: str,
                 definition: str,
                 function: str = None):
        """
        A constructor of FramesetArgument class which takes argumentType and definition as input and initializes
        corresponding attributes

        PARAMETERS
        ----------
        argumentType : str
            ArgumentType of the frameset argument
        definition : str
            Definition of the frameset argument
        function : str
            Function of the frameset argument
        """
        self.__argument_type = argumentType
        self.__definition = definition
        self.__function = function

    def getArgumentType(self) -> str:
        """
        Accessor for argumentType.

        RETURNS
        -------
        str
            argumentType.
        """
        return self.__argument_type

    def getDefinition(self) -> str:
        """
        Accessor for definition.

        RETURNS
        -------
        str
            definition.
        """
        return self.__definition

    def getFunction(self) -> str:
        """
        Accessor for function.

        RETURNS
        -------
        str
            function.
        """
        return self.__function

    def setDefinition(self, definition: str):
        """
        Mutator for definition.

        PARAMETERS
        ----------
        definition : str
            definition to set.
        """
        self.__definition = definition

    def setFunction(self, function: str):
        """
        Mutator for definition.

        PARAMETERS
        ----------
        function : str
            function to set.
        """
        self.__function = function

    def __str__(self) -> str:
        """
        __str__ converts an FramesetArgument to a string. It returns argument string which is in the form of
        argumentType:definition

        RETURNS
        -------
        str
            string form of frameset argument
        """
        return self.__argument_type + ":" + self.__definition



class Argument(object):

    __argument_type: str
    __id: str

    def constructor1(self, argument: str):
        """
        A constructor of Argument class which takes argument string which is in the form of argumentType$id
        and parses this string into argumentType and id. If the argument string does not contain '$' then the
        constructor return a NONE type argument.

        PARAMETERS
        ----------
        argument : str
            Argument string containing the argumentType and id
        """
        if "$" in argument:
            self.__argument_type = argument[0:argument.index("$")]
            self.__id = argument[argument.index("$") + 1:]
        else:
            self.__argument_type = "NONE"

    def constructor2(self, argumentType: str, _id: str):
        """
        Another constructor of Argument class which takes argumentType and id as inputs and initializes corresponding
        attributes

        PARAMETERS
        ----------
        argumentType : str
            Type of the argument
        _id : str
            Id of the argument
        """
        self.__argument_type = argumentType
        self.__id = _id

    def __init__(self, argumentOrType: str, _id: str = None):
        if _id is None:
            self.constructor1(argumentOrType)
        else:
            self.constructor2(argumentOrType, _id)

    def getArgumentType(self) -> str:
        """
        Accessor for argumentType.

        RETURNS
        -------
        str
            argumentType.
        """
        return self.__argument_type

    def getId(self) -> str:
        """
        Accessor for id.

        RETURNS
        -------
        str
            id.
        """
        return self.__id

    def setId(self, _id: str):
        """
        Setter for id.
        :param _id: New id
        """
        self.__id = _id

    def __str__(self) -> str:
        """
        __str__ converts an Argument to a string. If the argumentType is "NONE" returns only "NONE", otherwise
        it returns argument string which is in the form of argumentType$id

        RETURNS
        -------
        str
            string form of argument
        """
        if self.__argument_type == "NONE":
            return self.__argument_type
        else:
            return self.__argument_type + "$" + self.__id



from PropBank.RoleSet import RoleSet


class Predicate(object):

    __lemma: str
    __role_sets: list

    def __init__(self, lemma: str):
        """
        A constructor of Predicate class which takes lemma as input and initializes lemma with this input.
        The constructor also initializes the roleSets array.

        PARAMETERS
        ----------
        lemma : str
            Lemma of the predicate
        """
        self.__lemma = lemma
        self.__role_sets = []

    def getLemma(self) -> str:
        """
        Accessor for lemma.

        RETURNS
        -------
        str
            lemma.
        """
        return self.__lemma

    def addRoleSet(self, roleSet: RoleSet):
        """
        The addRoleSet method takes a RoleSet as input and adds it to the roleSets list.

        PARAMETERS
        ----------
        roleSet : RoleSet
            RoleSet to be added
        """
        self.__role_sets.append(roleSet)

    def size(self) -> int:
        """
        The size method returns the size of the roleSets list.

        RETURNS
        -------
        int
            the size of the roleSets list.
        """
        return len(self.__role_sets)

    def getRoleSet(self, index: int) -> RoleSet:
        """
        The getRoleSet method returns the roleSet at the given index.

        PARAMETERS
        ----------
        index : int
            Index of the roleSet

        RETURNS
        -------
        RoleSet
            RoleSet at the given index.
        """
        return self.__role_sets[index]

    def getRoleSetWithId(self, roleId: str) -> RoleSet:
        """
        Another getRoleSet method which returns the roleSet with the given roleSet id.

        PARAMETERS
        ----------
        roleId : str
            Id of the searched roleSet

        RETURNS
        -------
        RoleSet
            RoleSet which has the given id.
        """
        for role_set in self.__role_sets:
            if role_set.getId() == roleId:
                return role_set
        return None

    def __repr__(self):
        return f"{self.__lemma} {self.__role_sets}"



from setuptools import setup

from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text(encoding="utf-8")

setup(
    name='NlpToolkit-PropBank',
    version='1.0.22',
    packages=['PropBank', 'PropBank.data'],
    package_data={'PropBank.data': ['*.xml']},
    url='https://github.com/StarlangSoftware/TurkishPropbank-Py',
    license='',
    author='olcaytaner',
    author_email='olcay.yildiz@ozyegin.edu.tr',
    description='Turkish PropBank',
    long_description=long_description,
    long_description_content_type='text/markdown'
)



import os
import xml.etree.ElementTree
from collections import KeysView

import pkg_resources

from PropBank.Predicate import Predicate
from PropBank.RoleSet import RoleSet
from PropBank.Role import Role


class PredicateList(object):

    __list: dict

    def __init__(self, fileName = pkg_resources.resource_filename(__name__, 'data/english-propbank.xml')):
        """
        A constructor of PredicateList class which reads all predicate files inside the 'Frames' folder. For each
        file inside that folder, the constructor creates a Predicate and puts in inside the list dictionary.
        """
        self.__list = {}
        root = xml.etree.ElementTree.parse(fileName).getroot()
        for frame_set in root:
            for predicate in frame_set:
                lemma = predicate.attrib["lemma"]
                new_predicate = Predicate(lemma)
                for role_set in predicate:
                    _id = role_set.attrib["id"]
                    name = role_set.attrib["name"]
                    new_role_set = RoleSet(_id, name)
                    for roles in role_set:
                        for role in roles:
                            if "descr" in role.attrib:
                                descr = role.attrib["descr"]
                            else:
                                descr = ""
                            if "f" in role.attrib:
                                f = role.attrib["f"]
                            else:
                                f = ""
                            if "n" in role.attrib:
                                n = role.attrib["n"]
                            else:
                                n = ""
                            newRole = Role(descr, f, n)
                            new_role_set.addRole(newRole)
                    new_predicate.addRoleSet(new_role_set)
                self.__list[lemma] = new_predicate

    def size(self):
        """
        The size method returns the number of predicates inside the list.

        RETURNS
        -------
        int
            the size of the list dict.
        """
        return len(self.__list)

    def getPredicate(self, lemma: str) -> Predicate:
        """
        getPredicate method returns the Predicate with the given lemma.

        PARAMETERS
        ----------
        lemma : str
            Lemma of the searched predicate

        RETURNS
        -------
        Predicate
            Predicate which has the given lemma.
        """
        return self.__list[lemma]

    def getLemmaList(self) -> KeysView:
        """
        The method returns all lemma in the predicate list.

        RETURNS
        -------
        dict
            All lemma in the predicate list.
        """
        return self.__list.keys()



from PropBank.ArgumentType import ArgumentType


class Role(object):

    __description: str
    __f: str
    __n: str

    def __init__(self,
                 description: str,
                 f: str,
                 n: str):
        """
        A constructor of Role class which takes description, f, and n as inputs and initializes corresponding with
        these inputs.

        PARAMETERS
        ----------
        description : str
            Description of the role
        f : str
            Argument Type of the role
        n : str
            Number of the role
        """
        self.__description = description
        self.__f = f
        self.__n = n

    def getDescription(self) -> str:
        """
        Accessor for description.

        RETURNS
        -------
        str
            description.
        """
        return self.__description

    def getF(self) -> str:
        """
        Accessor for f.

        RETURNS
        -------
        str
            f.
        """
        return self.__f

    def getN(self) -> str:
        """
        Accessor for n.

        RETURNS
        -------
        str
            n.
        """
        return self.__n

    def getArgumentType(self) -> ArgumentType:
        """
        Constructs and returns the argument type for this role.

        RETURNS
        -------
        ArgumentType
            Argument type for this role.
        """
        return ArgumentType.getArguments("ARG" + self.__f.upper())

    def __repr__(self):
        return f"{self.__f} {self.__n} {self.__description}"



from xml.etree.ElementTree import Element

from PropBank.ArgumentType import ArgumentType
from PropBank.FramesetArgument import FramesetArgument


class Frameset(object):

    __frameset_arguments: list
    __id: str

    def __init__(self, framesetNode: Element = None):
        """
        Another constructor of Frameset class which takes filename as input and reads the frameset

        PARAMETERS
        ----------
        framesetNode : Element
            File name of the file to read frameset
        """
        if framesetNode is not None:
            self.__id = framesetNode.attrib["id"]
            self.__frameset_arguments = []
            for child in framesetNode:
                self.__frameset_arguments.append(FramesetArgument(child.attrib["name"], child.text,
                                                                  child.attrib["function"]))
        else:
            self.__id = ""
            self.__frameset_arguments = []

    def containsArgument(self, argumentType: ArgumentType) -> bool:
        """
        containsArgument method which checks if there is an Argument of the given argumentType.

        PARAMETERS
        ----------
        argumentType : ArgumentType
            ArgumentType of the searched Argument

        RETURNS
        -------
        bool
            true if the Argument with the given argumentType exists, false otherwise.
        """
        for frameset_argument in self.__frameset_arguments:
            if ArgumentType.getArguments(frameset_argument.getArgumentType()) == argumentType:
                return True
        return False

    def addArgument(self,
                    argumentType: str,
                    definition: str,
                    function: str = None):
        """
        The addArgument method takes a type and a definition of a FramesetArgument as input, then it creates a new
        FramesetArgument from these inputs and adds it to the framesetArguments list.

        PARAMETERS
        ----------
        argumentType : str
            Type of the new FramesetArgument
        definition : str
            Definition of the new FramesetArgument
        function: str
            Function of the new FramesetArgument
        """
        check = False
        for frameset_argument in self.__frameset_arguments:
            if frameset_argument.getArgumentType() == argumentType:
                frameset_argument.setDefinition(definition)
                check = True
                break
        if not check:
            arg = FramesetArgument(argumentType, definition, function)
            self.__frameset_arguments.append(arg)

    def deleteArgument(self,
                       argumentType: str,
                       definition: str):
        """
        The deleteArgument method takes a type and a definition of a FramesetArgument as input, then it searches for the
        FramesetArgument with these type and definition, and if it finds removes it from the framesetArguments list.

        PARAMETERS
        ----------
        argumentType : str
            Type of the to be deleted FramesetArgument
        definition : str
            Definition of the to be deleted FramesetArgument
        """
        for frameset_argument in self.__frameset_arguments:
            if frameset_argument.getArgumentType() == argumentType and frameset_argument.getDefinition() == definition:
                self.__frameset_arguments.remove(frameset_argument)
                break

    def getFramesetArguments(self) -> list:
        """
        Accessor for framesetArguments.

        RETURNS
        -------
        list
            framesetArguments.
        """
        return self.__frameset_arguments

    def getId(self) -> str:
        """
        Accessor for id.

        RETURNS
        -------
        str
            id.
        """
        return self.__id

    def setId(self, _id: str):
        """
        Mutator for id.

        PARAMETERS
        ----------
        _id : str
            id to set.
        """
        self.__id = _id

    def save(self, fileName: str):
        """
        Saves current frameset with the given filename.

        PARAMETERS
        ----------
        fileName : str
            Name of the output file.
        """
        output_file = open(fileName, mode="w", encoding="utf-8")
        output_file.write("<FRAMESET id=\"" + self.__id + "\">\n")
        for frameset_argument in self.__frameset_arguments:
            output_file.write("\t<ARG name=\"" + frameset_argument.getArgumentType() + "\" function=\"" +
                             frameset_argument.getFunction() + "\">" + frameset_argument.getDefinition() + "</ARG>\n")
        output_file.write("</FRAMESET>\n")
        output_file.close()

    def __repr__(self):
        return f"{self.__id} {self.__frameset_arguments}"
